#!/usr/bin/perl
###############################################################################
#
#	Copyright (C) 2003 Oy Realnode Ab
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
################################################################################
#
#   $Id: config.pl,v 1.5.4.3 2004/01/22 18:40:29 eb Exp $
#
################################################################################

use lib::MARC;
use lib::Config;

# Version declaration
$VERSION = '0.1.1';

# File::Basenam has been in Perl since 5.003, and nobody
# sane uses older, but one can never be too sure.
my $dir;
if ( eval q{require "File/Basename.pm"} ) {
    $dir = File::Basename::dirname($0);
    chdir($dir);
}

# we'll use Cwd to get the full directory path.
# the Basename stuff above will sometimes return '.' as
# the base directory, which is not helpful here.
use Cwd;
$dir = cwd();

# check for passed arguments
($current_config, $current_MARCsyn, $backups) = check_arguments();

# Then its time to read the data.
if((-e '../config') && $current_config eq "") {
	$current_config = '../config';
} elsif((-e '../config.example') && $current_config eq "") {
	$current_config = '../config.example';
} elsif($current_config eq "") {

	clear_screen();

	print "No configuration file found. Please obtain 'config.example'\n";
    print "or 'config' before running this again. This program needs\n";
    print "a default config file to get default values.\n";
	print "  If You did not get either of these config files in the Emilda\n";
	print "package You downloaded, please inform Us at emilda\@realnode.com\n";
    exit;

}

if(-e '../MARCsyn_config' && $current_MARCsyn eq "") {
	$current_MARCsyn = '../MARCsyn_config';
} elsif (-e '../MARCsyn_config.example' && $current_MARCsyn eq "") {
	$current_MARCsyn = '../MARCsyn_config.example';
}

%conf = lib::Config->conf_parser_advanced($current_config);
%MARCsyn = lib::Config->conf_parser_advanced($current_MARCsyn);


# We want to use colors by default
$use_color = 1;
$WHT = "\x1B[37;1m";
$RED = "\033[0;31m";
$GRN = "\033[1;32m";
$NRM = "\x1B[0m";

# check if we are runnin debug mode
if($debug_mode) {
	$mode = 'Debug';
    $config_out = '../config.debug';
	$MARCsyn_out = '../MARCsyn_config.debug';
} else {
	$mode = 'Normal';
    $config_out = '../config';
	$MARCsyn_out = '../MARCsyn_config';
}

if($backups) {
	$backups_note = '';
} else {
	$backups_note = '(Disable Backups)';
}

# Loop until the user breaks the loop
while (($command ne 'q') && ($command ne 'Q'))
{
	# Print menu structure
    clear_screen();
    print $WHT. "Emilda Configurator: " . $NRM;
	print " (Version: $VERSION)\n";
    print "---------------------------------------------------------\n";
    if($debug_mode) {
		print $WHT."Mode:$NRM $GRN$mode$NRM $backups_note\n";
	} else {
		print $WHT."Mode:$NRM $mode $backups_note\n";
	}
	print $WHT."Read:$NRM '$current_config', '$current_MARCsyn'\n";
	print $WHT."Write:$NRM '$config_out', '$MARCsyn_out'\n";
    print "---------------------------------------------------------\n";

    if ( $menu == 0 ) {
        print $WHT. "Main Menu\n" . $NRM;
        print "1.  MySQL Server Settings\n";
        print "2.  Zebra Server Settings\n";
        print "3.  Appearance Options\n";
        print "4.  System Options\n";
        print "5.  Security Options (Experts)\n";
		print "6.  Printer Options\n";
		print "7.  General Options\n";
        print "\n";
        print "D   Set Default Options";
        print "\n";
	}
	elsif ( $menu == 1) {
        print $WHT. "MySQL Server Settings\n" . $NRM;
        print "1.  MySQL User		: $WHT$conf{'mysql_user'}$NRM\n";
        print "2.  MySQL Password	: $WHT$conf{'mysql_passwd'}$NRM\n";
        print "3.  MySQL Database	: $WHT$conf{'mysql_db'}$NRM\n";
		print "4.  MySQL Host		: $WHT$conf{'mysql_host'}$NRM\n";
		print "5.  MySQL Port		: $WHT$conf{'mysql_port'}$NRM\n";
        print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 2) {
        print $WHT. "Zebra Server Settings\n" . $NRM;
        print "1.  Zebra Host            : $WHT$conf{'zebra_host'}$NRM\n";
        print "2.  Zebra Port            : $WHT$conf{'zebra_port'}$NRM\n";
        print "3.  Zebra Database        : $WHT$conf{'zebra_db'}$NRM\n";
        print "4.  Zebra Path            : $WHT$conf{'zebra_path'}$NRM\n";
        print "5.  Zebra Database Folder : $WHT$conf{'zebra_db_folder'}$NRM\n";
		print "6.  Zebra Config          : $WHT$conf{'zebra_config'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 3) {
        print $WHT. "Appearance Options\n" . $NRM;
        print "1.  Default Template      : $WHT$conf{'template'}$NRM\n";
        print "2.  Default Language      : $WHT$conf{'language'}$NRM\n";
        print "3.  Date Format           : $WHT$conf{'dateformat'}$NRM\n";
        print "4.  Time Format           : $WHT$conf{'timeformat'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 4) {
        print $WHT. "System Options\n" . $NRM;
        print "1.  Borrow Time               : $WHT$conf{'borrow_time'}$NRM\n";
        print "2.  Maximum Reborrow No.      : $WHT$conf{'reborrow_max'}$NRM\n";
        print "3.  Book ID Suffix            : $WHT$conf{'book_id_suffix'}$NRM\n";
        print "4.  Book ID Length            : $WHT$MARCsyn{'book_id_length'}$NRM\n";
        print "5.  Card Number Suffix        : $WHT$conf{'card_number_suffix'}$NRM\n";
        print "6.  Card Number Length        : $WHT$conf{'card_number_length'}$NRM\n";
		print "\n";
		print "$WHT!!  For Experts Only  !!$NRM\n";
		print "\n";
        print "7.  Control Number Identifier : $WHT$MARCsyn{'native_control_number_identifier'}$NRM\n";
        print "8.  Control Number Length     : $WHT$MARCsyn{'control_number_length'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 5) {
        print $WHT. "Security Options (Experts)\n" . $NRM;
        print "1.  Perl Link\n";
		print "      ID                       : $WHT$conf{'perl_id'}$NRM\n";
		print "      Auth                     : $WHT$conf{'perl_auth'}$NRM\n";
        print "2.  Debugging\n";
		print "      Trigger                  : $WHT$conf{'debug'}$NRM\n";
		print "      Debug Level              : $WHT$conf{'debug_level'}$NRM\n";
        print "3.  Session Timeout (s)        : $WHT$conf{'session_timeout'}$NRM\n";
        print "4.  Brutal Session Timeout (s) : $WHT$conf{'brutal_session_timeout'}$NRM\n";
        print "5.  Logging\n";
		print "      Trigger                  : $WHT$conf{'logging'}$NRM\n";
		print "      Log File                 : $WHT$conf{'log_file'}$NRM\n";
		print "6.  Emilda Distro\n";
		print "      Trigger                  : $WHT$conf{'using_distro'}$NRM\n";
		print "      User Agent               : $WHT$conf{'distro_user_agent'}$NRM\n";
		print "7.  IP Blocking\n";
		print "      Trigger                  : $WHT$conf{'ip_check'}$NRM\n";
		print "      Allowed IPs              : $WHT$conf{'ip_allowed'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 6) {
        print $WHT. "Printer Options\n" . $NRM;
        print "1.  Label Printer      : $WHT$conf{'label_printer'}$NRM\n";
        print "2.  Receipt Printer    : $WHT$conf{'receipt_printer'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}
	elsif ( $menu == 7) {
        print $WHT. "General Options\n" . $NRM;
        print "1.  SysAdmin e-Mail       : $WHT$conf{'admin_mail'}$NRM\n";
        print "2.  Developers e-Mail     : $WHT$conf{'developers_email'}$NRM\n";
        print "3.  Redirect Timeout      : $WHT$conf{'redirect_timeout'}$NRM\n";
        print "4.  Required User Info    : $WHT$conf{'user_required_fields'}$NRM\n";
        print "5.  Borrow Max\n";
		print "      Trigger             : $WHT$conf{'borrow_max'}$NRM\n";
		print "      Number              : $WHT$conf{'borrow_max_number'}$NRM\n";
		print "\n";
		print "R   Return to main menu\n";
	}


	if ( $use_color == 1 ) {
        print "C   Turn color off\n";
    } else {
        print "C   Turn color on\n";
    }
	print "H   Help\n";
    print "S   Save data\n";
    print "Q   Quit\n";

    print "\n";
    print "Choice >> " . $WHT;
    $command = <STDIN>;
    $command =~ s/[\n|\r]//g;
    $command =~ tr/A-Z/a-z/;
    print "$NRM\n";

    # Read the commands they entered.
    if ( $command eq "r" ) {
        $menu = 0;
    } elsif ( $command eq "s" ) {
        print "Do You really want to write changed data?\n";
        print "Save? [Y/".$WHT."n".$NRM."]: ";
        $save = <STDIN>;
        if (  $save =~ /^y/i ) {
            lib::Config->save_data($current_config, $current_MARCsyn, $config_out, $MARCsyn_out, $backups, %conf, %MARCsyn);
			$saved = 1;
			print $GRN."Data Saved Successfully\n".$NRM;
        } else {
            print $RED."Data Not saved!\n".$NRM;
		}
        print "Press enter to continue...";
        $tmp   = <STDIN>;
    } elsif ( ( $command eq "q" ) && ( $saved == 0 ) ) {
        print "You have not saved your data.\n";
        print "Save? [Y/".$WHT."n".$NRM."]: ";
        $save = <STDIN>;
        if (  $save =~ /^y/i ) {
            lib::Config->save_data($current_config, $current_MARCsyn, $config_out, $MARCsyn_out, $backups, %conf, %MARCsyn);
			print $GRN."Data Saved Successfully\n".$NRM;
        } else {
            print $RED."Data Not saved!\n".$NRM;
		}
    } elsif ( $command eq "c" ) {
        if ( $use_color == 1 ) {
            $use_color = 2;
            $WHT = "";
			$RED = "";
			$GRN = "";
            $NRM = "";
        } else {
            $use_color = 1;
            $WHT = "\x1B[37;1m";
			$RED = "\033[0;31m";
			$GRN = "\033[1;32m";
            $NRM = "\x1B[0m";
        }
    } elsif ( $command eq "d" && $menu == 0 ) {
        set_defaults();

	} elsif ($command eq 'h'){
		print_help();
	}
	else {
       $saved = 0;
        if ( $menu == 0 ) {
            if ( ( $command > 0 ) && ( $command < 8 ) ) {
                $menu = $command;
            }
		} elsif ($menu == 1){
            if    ( $command == 1 ) { $conf{'mysql_user'}		= com1(); }
            elsif ( $command == 2 ) { $conf{'mysql_passwd'}		= com2(); }
            elsif ( $command == 3 ) { $conf{'mysql_db'}  		= com3(); }
            elsif ( $command == 4 ) { $conf{'mysql_host'}		= com4(); }
            elsif ( $command == 5 ) { $conf{'mysql_port'}		= com5(); }
		} elsif ($menu == 2){
            if    ( $command == 1 ) { $conf{'zebra_host'}		= com21(); }
            elsif ( $command == 2 ) { $conf{'zebra_port'}		= com22(); }
            elsif ( $command == 3 ) { $conf{'zebra_db'}  		= com23(); }
            elsif ( $command == 4 ) { $conf{'zebra_path'}		= com24(); }
            elsif ( $command == 5 ) { $conf{'zebra_db_folder'}	= com25(); }
            elsif ( $command == 6 ) { $conf{'zebra_config'}		= com26(); }
		} elsif ($menu == 3){
            if    ( $command == 1 ) { $conf{'template'}		= com31(); }
            elsif ( $command == 2 ) { $conf{'language'}		= com32(); }
            elsif ( $command == 3 ) { $conf{'dateformat'}  	= com33(); }
            elsif ( $command == 4 ) { $conf{'timeformat'}	= com34(); }
		} elsif ($menu == 4){
            if    ( $command == 1 ) { $conf{'borrow_time'}							= com41(); }
            elsif ( $command == 2 ) { $conf{'reborrow_max'}							= com42(); }
            elsif ( $command == 3 ) { $conf{'book_id_suffix'}						= com43(); }
            elsif ( $command == 4 ) { $MARCsyn{'book_id_length'}					= com44(); }
            elsif ( $command == 5 ) { $conf{'card_number_suffix'}					= com45(); }
            elsif ( $command == 6 ) { $conf{'card_number_length'}					= com46(); }
            elsif ( $command == 7 ) { $MARCsyn{'native_control_number_identifier'}	= com47(); }
            elsif ( $command == 8 ) { $MARCsyn{'control_number_length'}				= com48(); }
		} elsif ($menu == 5){
            if    ( $command == 1 ) { ($conf{'perl_id'}, $conf{'perl_auth'})				= com51(); }
            elsif ( $command == 2 ) { ($conf{'debug'}, $conf{'debug_level'})				= com52(); }
            elsif ( $command == 3 ) { $conf{'session_timeout'}								= com53(); }
            elsif ( $command == 4 ) { $conf{'brutal_session_timeout'}						= com54(); }
            elsif ( $command == 5 ) { ($conf{'logging'}, $conf{'log_file'})					= com55(); }
			elsif ( $command == 6 ) { ($conf{'using_distro'}, $conf{'distro_user_agent'})	= com56(); }
            elsif ( $command == 7 ) { ($conf{'ip_check'}, $conf{'ip_allowed'})				= com57(); }
		} elsif ($menu == 6){
            if    ( $command == 1 ) { $conf{'label_printer'}	= com61(); }
            elsif ( $command == 2 ) { $conf{'receipt_printer'}	= com62(); }
		} elsif ($menu == 7){
            if    ( $command == 1 ) { $conf{'admin_mail'}								= com71(); }
            elsif ( $command == 2 ) { $conf{'developers_email'}							= com72(); }
            elsif ( $command == 3 ) { $conf{'redirect_timeout'}  						= com73(); }
            elsif ( $command == 4 ) { $conf{'user_required_fields'}						= com74(); }
            elsif ( $command == 5 ) { ($conf{'borrow_max'}, $conf{'borrow_max_number'})	= com75(); }
		}
	}
}

# Print exit message
clear_screen();
print $WHT. "Emilda Configurator: " . $NRM;
print " (Version: $VERSION)\n";
print "---------------------------------------------------------\n";
print "Thank You For Using Emilda Configurator!\n";
print "See You later, calculator.\n\n";

### COMMANDS ###

# mysql user
sub com1 {
	print "$conf{'mysql_desc'}\n";

	print "MySQL User [$WHT$conf{'mysql_user'}$NRM]: $WHT";
    $new_mysql_user = <STDIN>;
    if ( $new_mysql_user eq "\n" ) {
        $new_mysql_user = $conf{'mysql_user'};
    } else {
        $new_mysql_user =~ s/[\r|\n]//g;
        $new_mysql_user =~ s/^\s+$//g;
    }
    return $new_mysql_user;
}

# mysql password
sub com2 {
	print "$conf{'mysql_desc'}\n";

	print "MySQL Password [$WHT$conf{'mysql_passwd'}$NRM]: $WHT";
    $new_mysql_password = <STDIN>;
    if ( $new_mysql_password eq "\n" ) {
        $new_mysql_password = $conf{'mysql_passwd'};
    } else {
        $new_mysql_password =~ s/[\r|\n]//g;
        $new_mysql_password =~ s/^\s+$//g;
    }
    return $new_mysql_password;
}

# mysql database
sub com3 {
	print "$conf{'mysql_desc'}\n";

	print "MySQL Database [$WHT$conf{'mysql_db'}$NRM]: $WHT";
    $new_mysql_db = <STDIN>;
    if ( $new_mysql_db eq "\n" ) {
        $new_mysql_db = $conf{'mysql_db'};
    } else {
        $new_mysql_db =~ s/[\r|\n]//g;
        $new_mysql_db =~ s/^\s+$//g;
    }
    return $new_mysql_db;
}

# mysql host
sub com4 {
	print "$conf{'mysql_desc'}\n";

	print "MySQL Host [$WHT$conf{'mysql_host'}$NRM]: $WHT";
    $new_mysql_host = <STDIN>;
    if ( $new_mysql_host eq "\n" ) {
        $new_mysql_host = $conf{'mysql_host'};
    } else {
        $new_mysql_host =~ s/[\r|\n]//g;
        $new_mysql_host =~ s/^\s+$//g;
    }
    return $new_mysql_host;
}

# mysql port
sub com5 {
	print "$conf{'mysql_desc'}\n";

	print "MySQL Port [$WHT$conf{'mysql_port'}$NRM]: $WHT";
    $new_mysql_port = <STDIN>;
    if ( $new_mysql_port eq "\n" ) {
        $new_mysql_port = $conf{'mysql_port'};
    } else {
        $new_mysql_port =~ s/[\r|\n]//g;
        $new_mysql_port =~ s/^\s+$//g;
    }
    return $new_mysql_port;
}

# zebra host
sub com21 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Host [$WHT$conf{'zebra_host'}$NRM]: $WHT";
    $new_zebra_host = <STDIN>;
    if ( $new_zebra_host eq "\n" ) {
        $new_zebra_host = $conf{'zebra_host'};
    } else {
        $new_zebra_host =~ s/[\r|\n]//g;
        $new_zebra_host =~ s/^\s+$//g;
    }
    return $new_zebra_host;
}

# zebra port
sub com22 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Port [$WHT$conf{'zebra_port'}$NRM]: $WHT";
    $new_zebra_port = <STDIN>;
    if ( $new_zebra_port eq "\n" ) {
        $new_zebra_port = $conf{'zebra_port'};
    } else {
        $new_zebra_port =~ s/[\r|\n]//g;
        $new_zebra_port =~ s/^\s+$//g;
    }
    return $new_zebra_port;
}

# zebra database
sub com23 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Database [$WHT$conf{'zebra_db'}$NRM]: $WHT";
    $new_zebra_db = <STDIN>;
    if ( $new_zebra_db eq "\n" ) {
        $new_zebra_db = $conf{'zebra_db'};
    } else {
        $new_zebra_db =~ s/[\r|\n]//g;
        $new_zebra_db =~ s/^\s+$//g;
    }
    return $new_zebra_db;
}

# zebra path
sub com24 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Path [$WHT$conf{'zebra_path'}$NRM]: $WHT";
    $new_zebra_path = <STDIN>;
    if ( $new_zebra_path eq "\n" ) {
        $new_zebra_path = $conf{'zebra_path'};
    } else {
        $new_zebra_path =~ s/[\r|\n]//g;
        $new_zebra_path =~ s/^\s+$//g;
    }
    return $new_zebra_path;
}

# zebra database folder
sub com25 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Database [$WHT$conf{'zebra_db_folder'}$NRM]: $WHT";
    $new_zebra_db_folder = <STDIN>;
    if ( $new_zebra_db_folder eq "\n" ) {
        $new_zebra_db_folder = $conf{'zebra_db_folder'};
    } else {
        $new_zebra_db_folder =~ s/[\r|\n]//g;
        $new_zebra_db_folder =~ s/^\s+$//g;
    }
    return $new_zebra_db_folder;
}

# zebra config
sub com26 {
	print "$conf{'zebra_desc'}\n";

	print "Zebra Database [$WHT$conf{'zebra_config'}$NRM]: $WHT";
    $new_zebra_config = <STDIN>;
    if ( $new_zebra_config eq "\n" ) {
        $new_zebra_config = $conf{'zebra_config'};
    } else {
        $new_zebra_config =~ s/[\r|\n]//g;
        $new_zebra_config =~ s/^\s+$//g;
    }
    return $new_zebra_config;
}

# Template
sub com31 {
	print "$conf{'template_desc'}\n";

	print "Default Template [$WHT$conf{'template'}$NRM]: $WHT";
    $new_template = <STDIN>;
    if ( $new_template eq "\n" ) {
        $new_template = $conf{'template'};
    } else {
        $new_template =~ s/[\r|\n]//g;
        $new_template =~ s/^\s+$//g;
    }
    return $new_template;
}

# Language
sub com32 {
	print "$conf{'language_desc'}\n";

	print "Default Language [$WHT$conf{'language'}$NRM]: $WHT";
    $new_language = <STDIN>;
    if ( $new_language eq "\n" ) {
        $new_language = $conf{'language'};
    } else {
        $new_language =~ s/[\r|\n]//g;
        $new_language =~ s/^\s+$//g;
    }
    return $new_language;
}

# Date format
sub com33 {
	print "$conf{'dateformat_desc'}\n";

	print "Dateformat [$WHT$conf{'dateformat'}$NRM]: $WHT";
    $new_dateformat = <STDIN>;
    if ( $new_dateformat eq "\n" ) {
        $new_dateformat = $conf{'dateformat'};
    } else {
        $new_dateformat =~ s/[\r|\n]//g;
        #$new_dateformat =~ s/^\s+$//g;
    }
    return $new_dateformat;
}

# Time format
sub com34 {
	print "$conf{'dateformat_desc'}\n";

	print "Timeformat [$WHT$conf{'timeformat'}$NRM]: $WHT";
    $new_timeformat = <STDIN>;
    if ( $new_timeformat eq "\n" ) {
        $new_timeformat = $conf{'timeformat'};
    } else {
        $new_timeformat =~ s/[\r|\n]//g;
        #$new_timeformat =~ s/^\s+$//g;
    }
    return $new_timeformat;
}

# Borrow Time
sub com41 {
	print "$conf{'borrow_time_desc'}\n";

	print "New Borrow Time [$WHT$conf{'borrow_time'}$NRM]: $WHT";
    $new_borrow_time = <STDIN>;
    if ( $new_borrow_time eq "\n" ) {
        $new_borrow_time = $conf{'borrow_time'};
    } else {
        $new_borrow_time =~ s/[\r|\n]//g;
        $new_borrow_time =~ s/^\s+$//g;
    }
    return $new_borrow_time;
}

# Reborrow count
sub com42 {
	print "$conf{'reborrow_max_desc'}\n";

	print "New Max. Reborrow [$WHT$conf{'reborrow_max'}$NRM]: $WHT";
    $new_reborrow_max = <STDIN>;
    if ( $new_reborrow_max eq "\n" ) {
        $new_reborrow_max = $conf{'reborrow_max'};
    } else {
        $new_reborrow_max =~ s/[\r|\n]//g;
        $new_reborrow_max =~ s/^\s+$//g;
    }
    return $new_reborrow_max;
}

# Book id suffix
sub com43 {
	print "$conf{'book_id_suffix_desc'}\n";

	print "New Book ID [$WHT$conf{'book_id_suffix'}$NRM]: $WHT";
    $new_book_id_suffix = <STDIN>;
    if ( $new_book_id_suffix eq "\n" ) {
        $new_book_id_suffix = $conf{'book_id_suffix'};
    } else {
        $new_book_id_suffix =~ s/[\r|\n]//g;
        $new_book_id_suffix =~ s/^\s+$//g;
    }
    return $new_book_id_suffix;
}

# Book id length
sub com44 {
	print "$MARCsyn{'book_id_length_desc'}\n";

	print "New Book ID Suffix [$WHT$MARCsyn{'book_id_length'}$NRM]: $WHT";
    $new_book_id_length = <STDIN>;
    if ( $new_book_id_length eq "\n" ) {
        $new_book_id_length = $MARCsyn{'book_id_length'};
    } else {
        $new_book_id_length =~ s/[\r|\n]//g;
        $new_book_id_length =~ s/^\s+$//g;
    }
    return $new_book_id_length;
}

# Card number suffix
sub com45 {
	print "$conf{'card_number_suffix_desc'}\n";

	print "New Book ID Suffix [$WHT$conf{'card_number_suffix'}$NRM]: $WHT";
    $new_card_number_suffix = <STDIN>;
    if ( $new_card_number_suffix eq "\n" ) {
        $new_card_number_suffix = $conf{'card_number_suffix'};
    } else {
        $new_card_number_suffix =~ s/[\r|\n]//g;
        $new_card_number_suffix =~ s/^\s+$//g;
    }
    return $new_card_number_suffix;
}

# Card number length
sub com46 {
	print "$conf{'card_number_length_desc'}\n";

	print "New Book ID length [$WHT$conf{'card_number_length'}$NRM]: $WHT";
    $new_card_number_length = <STDIN>;
    if ( $new_card_number_length eq "\n" ) {
        $new_card_number_length = $conf{'card_number_length'};
    } else {
        $new_card_number_length =~ s/[\r|\n]//g;
        $new_card_number_length =~ s/^\s+$//g;
    }
    return $new_card_number_length;
}

# Native control number identifier
sub com47 {
	print "$MARCsyn{'native_control_number_identifier_desc'}\n";

	print "New Book ID length [$WHT$MARCsyn{'native_control_number_identifier'}$NRM]: $WHT";
    $new_native_control_number_identifier = <STDIN>;
    if ( $new_native_control_number_identifier eq "\n" ) {
        $new_native_control_number_identifier = $MARCsyn{'native_control_number_identifier'};
    } else {
        $new_native_control_number_identifier =~ s/[\r|\n]//g;
        $new_native_control_number_identifier =~ s/^\s+$//g;
    }
    return $new_native_control_number_identifier;
}

# Control number length
sub com48 {
	print "$MARCsyn{'control_number_length_desc'}\n";

	print "New Book ID length [$WHT$MARCsyn{'control_number_length'}$NRM]: $WHT";
    $new_control_number_length = <STDIN>;
    if ( $new_control_number_length eq "\n" ) {
        $new_control_number_length = $MARCsyn{'control_number_length'};
    } else {
        $new_control_number_length =~ s/[\r|\n]//g;
        $new_control_number_length =~ s/^\s+$//g;
    }
    return $new_control_number_length;
}

# Perl Link
sub com51
{
	print "$conf{'perl_desc'}\n";

	print "New Perl-Link ID [$WHT$conf{'perl_id'}$NRM]: $WHT";
	$new_perl_id = <STDIN>;

    if ( $new_perl_id eq "\n" ) {
        $new_perl_id = $conf{'perl_id'};
    } else {
        $new_perl_id =~ s/[\r|\n]//g;
        $new_perl_id =~ s/^\s+$//g;
    }

	print "$NRM";
	print "New Perl-Link Auth [$WHT$conf{'perl_auth'}$NRM]: $WHT";
	$new_perl_auth = <STDIN>;

    if ( $new_perl_auth eq "\n" ) {
        $new_perl_auth = $conf{'perl_auth'};
    } else {
        $new_perl_auth =~ s/[\r|\n]//g;
        $new_perl_auth =~ s/^\s+$//g;
    }

	return ($new_perl_id, $new_perl_auth);
}

# Debugging
sub com52
{
	$new_debug = 'nothing';
	$new_debug_level = 'nothing';

	print "$conf{'debug_desc'}\n";

	print "New Debug Type [$WHT$conf{'debug'}$NRM]: $WHT";

	until($new_debug eq '1' || $new_debug eq '0'){
		$new_debug = <STDIN>;

		if ( $new_debug eq "\n" ) {
			$new_debug = $conf{'debug'};
		} else {
			$new_debug =~ s/[\r|\n]//g;
			$new_debug =~ s/^\s+$//g;
		}
        if($new_debug != 1 && $new_debug != 0) {
			print $RED.'Incorrect Debug type: '.$WHT.$new_debug.$NRM;
			print "\n";
			print "New Debug Type [$WHT$conf{'debug'}$NRM]: $WHT";
		}
	}

	print "$NRM";
	print "New Debug Level [$WHT$conf{'debug_level'}$NRM]: $WHT";

	until($new_debug_level eq '0' || $new_debug_level eq '1' || $new_debug_level eq '2' || $new_debug_level eq '3' || $new_debug_level eq '4' || $new_debug_level eq '5') {
		$new_debug_level = <STDIN>;

		if ( $new_debug_level eq "\n" ) {
			$new_debug_level = $conf{'debug_level'};
		} else {
			$new_debug_level =~ s/[\r|\n]//g;
			$new_debug_level =~ s/^\s+$//g;
		}
        if($new_debug_level < 0 || $new_debug_level > 5) {
			print $RED.'Incorrect Debug Level: '.$WHT.$new_debug_level.$NRM;
			print "\n";
			print "New Debug Type [$WHT$conf{'debug_level'}$NRM]: $WHT";
		}
	}

	return ($new_debug, $new_debug_level);
}

# Session Timeout
sub com53 {
	print "$conf{'session_timeout_desc'}\n";

	print "New Session Timeout [$WHT$conf{'session_timeout'}$NRM]: $WHT";
    $new_session_timeout = <STDIN>;
    if ( $new_session_timeout eq "\n" ) {
        $new_session_timeout = $conf{'session_timeout'};
    } else {
        $new_session_timeout =~ s/[\r|\n]//g;
        $new_session_timeout =~ s/^\s+$//g;
    }
    return $new_session_timeout;
}

# Brutal Session Timeout
sub com54 {
	print "$conf{'brutal_session_timeout_desc'}\n";

	print "New brutal_session Timeout [$WHT$conf{'brutal_session_timeout'}$NRM]: $WHT";
    $new_brutal_session_timeout = <STDIN>;
    if ( $new_brutal_session_timeout eq "\n" ) {
        $new_brutal_session_timeout = $conf{'brutal_session_timeout'};
    } else {
        $new_brutal_session_timeout =~ s/[\r|\n]//g;
        $new_brutal_session_timeout =~ s/^\s+$//g;
    }
    return $new_brutal_session_timeout;
}

# Logging
sub com55
{
	$new_logging = 'nothing';
	$new_logging_file = 'nothing';

	print "$conf{'logging_desc'}\n";

	print "New Logging Type [$WHT$conf{'logging'}$NRM]: $WHT";

	until($new_logging eq '1' || $new_logging eq '0'){
		$new_logging = <STDIN>;

		if ( $new_logging eq "\n" ) {
			$new_logging = $conf{'logging'};
		} else {
			$new_logging =~ s/[\r|\n]//g;
			$new_logging =~ s/^\s+$//g;
		}
        if($new_logging != 1 && $new_logging != 0) {
			print $RED.'Incorrect logging type: '.$WHT.$new_logging.$NRM;
			print "\n";
			print "New logging Type [$WHT$conf{'logging'}$NRM]: $WHT";
		}
	}

	print "$NRM";
	print "New Logging File [$WHT$conf{'log_file'}$NRM]: $WHT";

	$new_logging_file = <STDIN>;

	if ( $new_logging_file eq "\n" ) {
		$new_logging_file = $conf{'log_file'};
	} else {
		$new_logging_file =~ s/[\r|\n]//g;
		$new_logging_file =~ s/^\s+$//g;
	}

	return ($new_logging, $new_logging_file);
}

# Distro
sub com56
{
	$new_using_distro = 'nothing';
	$new_distro_user_agent = 'nothing';

	print "$conf{'distro_and_ip-check_desc'}\n";

	print "New Distro Setting [$WHT$conf{'using_distro'}$NRM]: $WHT";

	until($new_using_distro eq '1' || $new_using_distro eq '0'){
		$new_using_distro = <STDIN>;

		if ( $new_using_distro eq "\n" ) {
			$new_using_distro = $conf{'using_distro'};
		} else {
			$new_using_distro =~ s/[\r|\n]//g;
			$new_using_distro =~ s/^\s+$//g;
		}
        if($new_using_distro != 1 && $new_using_distro != 0) {
			print $RED.'Incorrect setting: '.$WHT.$new_using_distro.$NRM;
			print "\n";
			print "New Distro Setting [$WHT$conf{'using_distro'}$NRM]: $WHT";
		}
	}

	print "$NRM";
	print "New Distro User Agent [$WHT$conf{'distro_user_agent'}$NRM]: $WHT";

	$new_distro_user_agent = <STDIN>;

	if ( $new_distro_user_agent eq "\n" ) {
		$new_distro_user_agent = $conf{'distro_user_agent'};
	} else {
		$new_distro_user_agent =~ s/[\r|\n]//g;
		#$new_distro_user_agent =~ s/^\s+$//g;
	}

	return ($new_using_distro, $new_distro_user_agent);
}

# IP blocking
sub com57
{
	$new_ip_check = 'nothing';
	$new_ip_allowed = 'nothing';

	print "$conf{'ip_check_desc'}\n";

	print "New IP Block Type [$WHT$conf{'ip_check'}$NRM]: $WHT";

	until($new_ip_check eq '1' || $new_ip_check eq '0'){
		$new_ip_check = <STDIN>;

		if ( $new_ip_check eq "\n" ) {
			$new_ip_check = $conf{'ip_check'};
		} else {
			$new_ip_check =~ s/[\r|\n]//g;
			$new_ip_check =~ s/^\s+$//g;
		}
        if($new_ip_check != 1 && $new_ip_check != 0) {
			print $RED.'Incorrect IP block type: '.$WHT.$new_ip_check.$NRM;
			print "\n";
			print "New IP Block Type [$WHT$conf{'ip_check'}$NRM]: $WHT";
		}
	}

	print "$NRM";
	print "Allowed IPs [$WHT$conf{'ip_allowed'}$NRM]: $WHT";

	$new_ip_allowed = <STDIN>;

	if ( $new_ip_allowed eq "\n" ) {
		$new_ip_allowed = $conf{'ip_allowed'};
	} else {
		$new_ip_allowed =~ s/[\r|\n]//g;
		$new_ip_allowed =~ s/^\s+$//g;
	}

	return ($new_ip_check, $new_ip_allowed);
}

# Label printer
sub com61
{
	$new_label_printer = 'nothing';

	print "$conf{'printers_desc'}\n";

	print "New Label Printer Setting [$WHT$conf{'label_printer'}$NRM]: $WHT";

	until($new_label_printer eq '1' || $new_label_printer eq '0'){
		$new_label_printer = <STDIN>;

		if ( $new_label_printer eq "\n" ) {
			$new_label_printer = $conf{'label_printer'};
		} else {
			$new_label_printer =~ s/[\r|\n]//g;
			$new_label_printer =~ s/^\s+$//g;
		}
        if($new_label_printer != 1 && $new_label_printer != 0) {
			print $RED.'Incorrect setting: '.$WHT.$new_label_printer.$NRM;
			print "\n";
			print "New Label Printer Setting [$WHT$conf{'label_printer'}$NRM]: $WHT";
		}
	}

	return ($new_label_printer);
}

# Receipt printer
sub com62
{
	$new_receipt_printer = 'nothing';

	print "$conf{'printers_desc'}\n";

	print "New Receipt Printer Setting [$WHT$conf{'receipt_printer'}$NRM]: $WHT";

	until($new_receipt_printer eq '1' || $new_receipt_printer eq '0'){
		$new_receipt_printer = <STDIN>;

		if ( $new_receipt_printer eq "\n" ) {
			$new_receipt_printer = $conf{'receipt_printer'};
		} else {
			$new_receipt_printer =~ s/[\r|\n]//g;
			$new_receipt_printer =~ s/^\s+$//g;
		}
        if($new_receipt_printer != 1 && $new_receipt_printer != 0) {
			print $RED.'Incorrect setting: '.$WHT.$new_receipt_printer.$NRM;
			print "\n";
			print "New Receipt Printer Setting [$WHT$conf{'receipt_printer'}$NRM]: $WHT";
		}
	}

	return ($new_receipt_printer);
}

# Admin email
sub com71 {
	print "$conf{'admin_mail_desc'}\n";

	print "New SysAdmin e-Mail [$WHT$conf{'admin_mail'}$NRM]: $WHT";
    $new_admin_email = <STDIN>;
    if ( $new_admin_email eq "\n" ) {
        $new_admin_email = $conf{'admin_mail'};
    } else {
        $new_admin_email =~ s/[\r|\n]//g;
        $new_admin_email =~ s/^\s+$//g;
    }
    return $new_admin_email;
}

# Developers email
sub com72 {
	print "$conf{'developers_email_desc'}\n";

	print "New Developers e-Mail [$WHT$conf{'developers_email'}$NRM]: $WHT";
    $new_developers_email = <STDIN>;
    if ( $new_developers_email eq "\n" ) {
        $new_developers_email = $conf{'developers_email'};
    } else {
        $new_developers_email =~ s/[\r|\n]//g;
        $new_developers_email =~ s/^\s+$//g;
    }
    return $new_developers_email;
}

# Redirect Timeout
sub com73 {
	print "$conf{'redirect_timeout_desc'}\n";

	print "New Redirect Timeout [$WHT$conf{'redirect_timeout'}$NRM]: $WHT";
    $new_redirect_timeout = <STDIN>;
    if ( $new_redirect_timeout eq "\n" ) {
        $new_redirect_timeout = $conf{'redirect_timeout'};
    } else {
        $new_redirect_timeout =~ s/[\r|\n]//g;
        $new_redirect_timeout =~ s/^\s+$//g;
    }
    return $new_redirect_timeout;
}

# Required user info
sub com74 {
	print "$conf{'user_required_fields_desc'}\n";

	print "Required User Information [$WHT$conf{'user_required_fields'}$NRM]: $WHT";
    $new_user_required_fields = <STDIN>;
    if ( $new_user_required_fields eq "\n" ) {
        $new_user_required_fields = $conf{'user_required_fields'};
    } else {
        $new_user_required_fields =~ s/[\r|\n]//g;
        $new_user_required_fields =~ s/^\s+$//g;
    }
    return $new_user_required_fields;
}

# Borrow max
sub com75
{
	print "$conf{'borrow_max_desc'}\n";

	print "New Borrow Max Type [$WHT$conf{'borrow_max'}$NRM]: $WHT";

	until($new_borrow_max eq '1' || $new_borrow_max eq '0'){
		$new_borrow_max = <STDIN>;

		if ( $new_borrow_max eq "\n" ) {
			$new_borrow_max = $conf{'borrow_max'};
		} else {
			$new_borrow_max =~ s/[\r|\n]//g;
			$new_borrow_max =~ s/^\s+$//g;
		}
        if($new_borrow_max != 1 && $new_borrow_max != 0) {
			print $RED.'Incorrect Borrow max type: '.$WHT.$new_borrow_max.$NRM;
			print "\n";
			print "New Borrow Max Type [$WHT$conf{'borrow_max'}$NRM]: $WHT";
		}
	}

	print "$NRM";
	print "Number of Books [$WHT$conf{'borrow_max_number'}$NRM]: $WHT";

	$new_borrow_max_number = <STDIN>;

	if ( $new_borrow_max_number eq "\n" ) {
		$new_borrow_max_number = $conf{'borrow_max_number'};
	} else {
		$new_borrow_max_number =~ s/[\r|\n]//g;
		$new_borrow_max_number =~ s/^\s+$//g;
	}

	return ($new_borrow_max, $new_borrow_max_number);
}



sub set_defaults {
	if(-e '../config.example') {
		$default_conf = '../config.example';
	} elsif (-e '../config') {
		$default_conf = '../config';
	}

	if(-e '../MARC_config.example') {
		$default_MARC = '../MARC_config.example';
	} elsif (-e '../MARC_config') {
		$default_MARC = '../MARC_config';
	}

	if(-e '../MARCsyn_config.example') {
		$default_MARCsyn = '../MARCsyn_config.example';
	} elsif (-e '../MARCsyn_config') {
		$default_MARCsyn = '../MARCsyn_config';
	}

	%conf = lib::Config->conf_parser_advanced($default_conf);
	%MARC = lib::Config->conf_parser_advanced($default_MARC);
	%MARCsyn = lib::Config->conf_parser_advanced($default_MARCsyn);
}

sub check_arguments
{
	# Predeclare some variables
	$current_config = "";
	$current_MARCsyn = "";
	$backups = TRUE;
    $i=0;

	# While loop to parse through the argument list
	while ($i <= $#ARGV)
	{
		if($ARGV[$i] eq '--debug-mode' || $ARGV[$i] eq '-d') {
			$debug_mode = TRUE;
		} elsif ($ARGV[$i] =~ /^--config-in=/) {
			print "HERE\n";
			$current_config = (split(/=/, $ARGV[$i]))[1];
		} elsif ($ARGV[$i] eq '-c') {
			if($ARGV[$i+1] =~ /^-/) {
				$help = TRUE;
				$i = $#ARGV;
			} else {
				$current_config = $ARGV[$i+1];
			}
			$i++;
		} elsif ($ARGV[$i] =~ /^--MARCsyn-in=/) {
			$current_MARCsyn = (split(/=/, $ARGV[$i]))[1];
		} elsif ($ARGV[$i] eq '-m') {
			if($ARGV[$i+1] =~ /^-/) {
				$help = TRUE;
				$i = $#ARGV;
			} else {
				$current_MARCsyn = $ARGV[$i+1];
			}
			$i++;
		} elsif ($ARGV[$i] eq '--disable-backups' || $ARGV[$i] eq '-b') {
			$backups = FALSE;
		} elsif ($ARGV[$i] eq '--help' || $ARGV[$i] eq '-h') {
			$help = TRUE;
		} else {
			$help = TRUE;
		}
		$i++;
	}

	# Check if passed arguments exist
	if($current_config ne "" && !(-e $current_config)) {
		print "Given --config-in Value does not exist: $current_config\n";
		exit;
	}

	if($current_MARCsyn ne "" && !(-e $current_MARCsyn)) {
		print "Given --MARCsyn-in Value does not exist: $current_MARCsyn\n";
		exit;
	}

	# If help is asked for, print it!
	if($help) {
		print "Usage: config.pl [OPTION]\n\n";
		print "Tool to configure the Emilda system\n\n";
		print " -c, --config-in=FILE     Specify which config to parse as working\n";
		print "                          config.\n";
		print " -m, --MARCsyn-in=FILE    Specify which MARCsyn_config to parse as\n";
		print "                          working config.\n";
		print " -b, --disable-backups    Disable backp files of old config, which are\n";
		print "                          written in the same folder as the configs.\n";
		print " -d, --debug-mode         Enable debug mode of the Emilda configurator.\n";
		print "                          Enabling this will not overwrite the actual\n";
		print "                          configuration files in the Emilda source folder.\n";
		print " -h, --help               This Help.\n\n\n";
		exit;
	}

	# Return, if appropriate
	return ($current_config, $current_MARCsyn)
}

sub print_help {
	print $WHT."Quick Help\n".$NRM;
	print "Use the digits on the left side of the text to navigate\n";
	print "or to set values of different variables.\n";
	print " When setting variables follow the on-screen description\n";
	print "printed under the menu.\n";
	print " Save data before exiting by typing 's' or 'S' in any menu.\n\n";
	print "Press enter to go back...";
	$tmp = <STDIN>;
}

sub clear_screen
{
    #Check if we are running on a MS platform
	if ( $^O =~ /^mswin/i) {
        system "cls";
    } else {
        system "clear";
    }
}
